@@ -175,7 +175,7 @@ class Data_inegrator:
                 else:
                     field1 = patient.field1
                     latent_representation = full_vae.yvae(field1)[4]
-                latent_representation_list.append(latent_representation.numpy())
+                latent_representation_list.append(latent_representation.squeeze().numpy())
                 patient_id_list.append(patient.id)
             latent_representation_df = pd.DataFrame(data=latent_representation_list, index=patient_id_list)
             latent_representation_df.to_csv(os.path.join(self.results_path, 'latent_representation.csv'))
